diff -ruN -x configure -x *~ -x ircd-hybrid-6.0.orig -x *.rej ircd-hybrid-6.0.orig/include/client.h ircd-hybrid-6.0/include/client.h
--- ircd-hybrid-6.0.orig/include/client.h	Sun Jun  3 14:05:51 2001
+++ ircd-hybrid-6.0/include/client.h	Sun Jun  3 13:56:31 2001
@@ -392,6 +392,11 @@
 
 #define SEND_UMODES  (FLAGS_INVISIBLE | FLAGS_OPER | FLAGS_WALLOP | \
                       FLAGS_IMMUNE)
+/*
+ * Only opers can have modes outside USER_UMODES.  If you change this
+ * set, then also change the explanatory NOTICE in user_mode(). -dark
+ */
+#define USER_UMODES  (FLAGS_INVISIBLE | FLAGS_WALLOP)
 #define ALL_UMODES   (SEND_UMODES | FLAGS_SERVNOTICE | FLAGS_CCONN | \
                       FLAGS_REJ | FLAGS_SKILL | FLAGS_FULL | FLAGS_SPY | \
                       FLAGS_NCHANGE | FLAGS_OPERWALL | FLAGS_DEBUG | \
diff -ruN -x configure -x *~ -x ircd-hybrid-6.0.orig -x *.rej ircd-hybrid-6.0.orig/include/config.h ircd-hybrid-6.0/include/config.h
--- ircd-hybrid-6.0.orig/include/config.h	Sun Jun  3 14:05:51 2001
+++ ircd-hybrid-6.0/include/config.h	Sun Jun  3 14:05:29 2001
@@ -460,6 +460,11 @@
 #define OPER_UMODES   (FLAGS_OPER|FLAGS_WALLOP|FLAGS_SERVNOTICE|FLAGS_OPERWALL|FLAGS_SKILL|FLAGS_BOTS)
 #define LOCOP_UMODES   (FLAGS_LOCOP|FLAGS_WALLOP|FLAGS_SERVNOTICE|FLAGS_OPERWALL|FLAGS_REJ|FLAGS_SKILL|FLAGS_BOTS|FLAGS_SPY)
 
+/* USE_USER_UMODES
+ * This prevents non-opers from ever gaining any umodes not set in USER_UMODES
+ */
+#define USE_USER_UMODES
+
 /* MAXIMUM LINKS - max links for class 0 if no Y: line configured
  *
  * This define is useful for leaf nodes and gateways. It keeps you from
diff -ruN -x configure -x *~ -x ircd-hybrid-6.0.orig -x *.rej ircd-hybrid-6.0.orig/src/s_user.c ircd-hybrid-6.0/src/s_user.c
--- ircd-hybrid-6.0.orig/src/s_user.c	Sun Jun  3 14:05:51 2001
+++ ircd-hybrid-6.0/src/s_user.c	Sun Jun  3 13:56:31 2001
@@ -2225,6 +2225,9 @@
 
               Count.oper--;
 
+#ifdef USE_USER_UMODES 
+	      acptr->umodes &= USER_UMODES | ~ALL_UMODES;
+#endif
               if (MyConnect(acptr))
                 {
                   aClient *prev_cptr = (aClient *)NULL;
@@ -2287,6 +2290,17 @@
             }
           break;
         }
+
+#ifdef USE_USER_UMODES
+  /* Use ALL_UMODES to make sure we don't mess with special server flags. 
+   * And (acptr->umodes ~setflags) to prevent umodes set by an oper from
+   * being removed */
+  if (((acptr->umodes & ~setflags) & ALL_UMODES & ~USER_UMODES) && !IsAnOper(sptr))
+    {
+      acptr->umodes &= USER_UMODES | ~ALL_UMODES;
+      badflag = YES;
+    }
+#endif
 
   if(badflag)
     sendto_one(sptr, form_str(ERR_UMODEUNKNOWNFLAG), me.name, parv[0]);
